import os
import base64
from typing import Optional
from dataclasses import dataclass
from pathlib import Path

from ..utils import declare_component


_component_func = declare_component(Path(__file__).absolute())


@dataclass
class AudioInputComponentOutput:
    data: bytes
    mime_type: str

# Create a wrapper function for the component. This is an optional
# best practice - we could simply expose the component function returned by
# `declare_component` and call it done. The wrapper allows us to customize
# our component's API: we can pre-process its input args, post-process its
# output value, and add a docstring for users.
def audio_input(key=None) -> Optional[AudioInputComponentOutput]:
    """TODO: DOC
    """

    component_data = _component_func(key=key, default=None)
    if component_data is None:
        return None
    message_bytes = component_data['b64'].encode('ascii')
    base64_bytes = base64.b64decode(message_bytes)
    return AudioInputComponentOutput(
        data=base64_bytes,
        mime_type=component_data['mime_type']
    )



import os
import base64
from io import BytesIO
from pathlib import Path
import streamlit.components.v1 as components
from typing import Optional, Any, Dict

from ..utils import image_to_url, declare_component


_component_func = declare_component(Path(__file__).absolute())


class ImageCropResult:
    def __init__(self, input_image: Any, component_value: Dict[str, Any]) -> None:
        self._component_value = component_value
        self._input_image = input_image

    @property
    def x(self) -> int:
        return int(self._component_value["x"])
    @property
    def y(self) -> int:
        return int(self._component_value["y"])
    @property
    def width(self) -> int:
        return int(self._component_value["width"])
    @property
    def height(self) -> int:
        return int(self._component_value["height"])


def image_crop(image: Any, previewResult: bool = False, key=None) -> Optional[ImageCropResult]:
    """Create a new instance of "my_component".

    Parameters
    ----------
    image: str or PIL.Image
        The image to crop. Can be either a URL or a PIL.Image
    previewResult: bool
        If the component should also render the selected crop
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.

    Returns
    -------
    ImageCropResult or None

    """
    # Call through to our private component function. Arguments we pass here
    # will be sent to the frontend, where they'll be available in an "args"
    # dictionary.
    #
    # "default" is a special argument that specifies the initial return
    # value of the component before the user has interacted with it.
    component_value = _component_func(image=image_to_url(image), previewResult=previewResult, key=key, default=None)
    if component_value is None:
        return None
    return ImageCropResult(image, component_value)



from io import BytesIO
import base64
import os
from typing import Any
from pathlib import Path
from threading import Lock
import streamlit.components.v1 as components


def image_to_url(image: Any) -> str:
    # URL
    if isinstance(image, str):
        return image
    # Pillow image
    import PIL.Image
    if isinstance(image, PIL.Image.Image):
        buffered = BytesIO()
        image.convert("RGB").save(buffered, format="JPEG")
        img_str = base64.b64encode(buffered.getvalue())
        r = bytes("data:image/jpeg;base64,", encoding='utf-8') + img_str
        return r.decode("utf-8")
    raise RuntimeError(f"Unsupported image type: {type(image)}. Provide a string (URL) or a PIL.Image")


def declare_component(path: Path):
    # Declare a Streamlit component. `declare_component` returns a function
    # that is used to create instances of the component. We're naming this
    # function "_component_func", with an underscore prefix, because we don't want
    # to expose it directly to users. Instead, we will create a custom wrapper
    # function, below, that will serve as our component's public API.

    # It's worth noting that this call to `declare_component` is the
    # *only thing* you need to do to create the binding between Streamlit and
    # your component frontend. Everything else we do in this file is simply a
    # best practice.

    component_name = path.parent.name
    serve_url = os.environ.get(f"{component_name.upper()}_URL", None)
    print("serve_url", f"{component_name.upper()}_URL", serve_url)


    if serve_url is not None:
        return components.declare_component(
            # We give the component a simple, descriptive name ("my_component"
            # does not fit this bill, so please choose something better for your
            # own component :)
            component_name,
            # Pass `url` here to tell Streamlit that the component will be served
            # by the local dev server that you run via `npm run start`.
            # (This is useful while your component is in development.)
            url=serve_url,
        )
    else:
        # When we're distributing a production version of the component, we'll
        # replace the `url` param with `path`, and point it to to the component's
        # build directory:
        build_dir = str(path.parent / "build")
        return components.declare_component(component_name, path=build_dir)


def _get_global_object(name, default_ctor):
    from matplotlib.backends.backend_agg import RendererAgg
    import sys
    GLOBAL_CONTAINER = sys

    if not hasattr(GLOBAL_CONTAINER, '_global_state'):
        with RendererAgg.lock:
            if not hasattr(GLOBAL_CONTAINER, '_global_state'):
                GLOBAL_CONTAINER._global_state = {}
                GLOBAL_CONTAINER._global_lock = Lock()

    if name not in GLOBAL_CONTAINER._global_state:
        with GLOBAL_CONTAINER._global_lock:
            if name not in GLOBAL_CONTAINER._global_state:
                GLOBAL_CONTAINER._global_state[name] = default_ctor()

    return GLOBAL_CONTAINER._global_state[name]


def critical_section(name: str):
    lock = _get_global_object(f"{name}.lock", Lock)
    def make_locked(func):
        def inner(*args, **kwargs):
            with lock:
                return func(*args, **kwargs)
        return inner
    return make_locked



import os
from pathlib import Path

from ..utils import declare_component


_component_func = declare_component(Path(__file__).absolute())


# Create a wrapper function for the component. This is an optional
# best practice - we could simply expose the component function returned by
# `declare_component` and call it done. The wrapper allows us to customize
# our component's API: we can pre-process its input args, post-process its
# output value, and add a docstring for users.
def template_component(name, key=None):
    """Create a new instance of "my_component".

    Parameters
    ----------
    name: str
        The name of the thing we're saying hello to. The component will display
        the text "Hello, {name}!"
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.

    Returns
    -------
    int
        The number of times the component's "Click Me" button has been clicked.
        (This is the value passed to `Streamlit.setComponentValue` on the
        frontend.)

    """
    # Call through to our private component function. Arguments we pass here
    # will be sent to the frontend, where they'll be available in an "args"
    # dictionary.
    #
    # "default" is a special argument that specifies the initial return
    # value of the component before the user has interacted with it.
    component_value = _component_func(name=name, key=key, default=0)

    # We could modify the value returned from the component if we wanted.
    # There's no need to do this in our simple example - but it's an option.
    return component_value



import setuptools
import os

METASTREAMLIT_VERSION = os.environ.get("METASTREAMLIT_VERSION", "0.0.0")

setuptools.setup(
    name="metastreamlit",
    version=METASTREAMLIT_VERSION,
    author="Daniel Haziza",
    author_email="",
    description="Helper library for creating ML demos in streamlit",
    long_description="",
    long_description_content_type="text/plain",
    url="",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[],
    python_requires=">=3.6",
    install_requires=[
        # By definition, a Custom Component depends on Streamlit.
        # If your component has other Python dependencies, list
        # them here.
        "streamlit >= 1.9",
        "matplotlib",
    ],
)



import os
import base64
from io import BytesIO
from pathlib import Path
import streamlit.components.v1 as components
from typing import Optional, Any, Dict, Sequence
from dataclasses import dataclass


from ..utils import image_to_url, declare_component


_component_func = declare_component(Path(__file__).absolute())

@dataclass
class OverlayPosition:
    """
    Position and dimensions of the overlay (in pixels)
    """
    x: int
    y: int
    width: int
    height: int

class ImagePositionOverlaysResult:
    def __init__(self, component_value: Dict[str, Any], origin: str) -> None:
        self._component_value = component_value
        self.origin = origin

    def __len__(self) -> int:
        return len(self._component_value['positions'])

    def __getitem__(self, idx: int) -> Optional[OverlayPosition]:
        if not isinstance(idx, int) or idx >= len(self) or idx < -len(self):
            raise KeyError(f"Invalid key: {idx}")
        pos_dict = self._component_value['positions'][idx]
        if pos_dict is None:
            return None
        return OverlayPosition(
            x=int(pos_dict['x']),
            y=int(pos_dict['y']),
            height=int(pos_dict['height']),
            width=int(pos_dict['width']),
        )


def image_position_overlays(background: Any, overlays: Sequence[Any], initialPositions: Optional[Sequence[Optional[Dict[str, Any]]]] = None, key=None, **kwargs: Any) -> ImagePositionOverlaysResult:
    """Create a new instance of "my_component".

    Parameters
    ----------
    background: str or PIL.Image
        The background image on which we want to position an overlay
    overlays: List[str or PIL.Image]
        Overlay images
    initialPositions:
        Initial positions for the overlays. Each position is a Dict with this format:
        {
            x: 25,
            y: 25,
            width: 50,
            height: 50
        }
    **kwargs: additional arguments are forwarded to the component.
        See https://github.com/DominicTobias/react-image-crop#props
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.

    Returns
    -------
    ImagePositionOverlaysResult or None

    """

    assert len(overlays) > 0, "Must provide at least one overlay"
    if len(overlays) > 1:
        raise NotImplementedError("`image_position_overlays` only supports a single overlay at the moment")
    if initialPositions is not None:
        assert len(initialPositions) == len(overlays), f"Provided {len(overlays)} overlays, but {len(initialPositions)} initial positions"
    if initialPositions is None:
        initialPositions = [None] * len(overlays)

    component_value = _component_func(
        background=image_to_url(background),
        overlays=[image_to_url(overlay) for overlay in overlays],
        initialPositions=initialPositions,
        kwargs=kwargs,
        key=key,
        default=None
    )
    if component_value is None:
        return ImagePositionOverlaysResult({'positions': initialPositions}, origin="default")
    return ImagePositionOverlaysResult(component_value, origin="component")



import os
import base64
from io import BytesIO
import streamlit.components.v1 as components
from typing import Optional, Dict, Any, Sequence
from dataclasses import dataclass
from pathlib import Path

from ..utils import image_to_url, declare_component


_component_func = declare_component(Path(__file__).absolute())


class ImageInputResult:
    def __init__(self, *, component_value: Optional[Dict[str, Any]] = None, input_pil_image = None) -> None:
        self._component_value = component_value
        self._input_pil_image = input_pil_image

    def get_pil_image(self) -> Any:
        import PIL.Image # Make it an optional dependency
        if self._component_value is None:
            return self._input_pil_image
        return PIL.Image.open(BytesIO(base64.b64decode(self._component_value["b64"])))

    @property
    def origin(self) -> str:
        if self._component_value is None:
            return "default"
        return self._component_value["origin"]


def image_input(initialDefaultPicture: Optional[str] = None, maxPixels: int = 512 * 512, presetImages: Optional[Sequence[str]] = None, key=None) -> Optional[ImageInputResult]:
    """Create a new instance of "my_component".

    Parameters
    ----------
    initialDefaultPictureURL: str or None
        URL of the initially selected picture
    maxPixels: int
        Maximum number of pixels in the uploaded image. Will be down-scaled if necessary
    presetImages: List of str or NOne
        A list of preset images to choose from
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.

    Returns
    -------
    int
        The number of times the component's "Click Me" button has been clicked.
        (This is the value passed to `Streamlit.setComponentValue` on the
        frontend.)

    """
    # Call through to our private component function. Arguments we pass here
    # will be sent to the frontend, where they'll be available in an "args"
    # dictionary.
    #
    # "default" is a special argument that specifies the initial return
    # value of the component before the user has interacted with it.
    component_value = _component_func(
        initialDefaultPicture=image_to_url(initialDefaultPicture),
        maxPixels=maxPixels,
        presetImages=[image_to_url(i) for i in presetImages] if presetImages is not None else [],
        key=key,
        default=None
    )
    if component_value is None:
        if not isinstance(initialDefaultPicture, str):
            return ImageInputResult(input_pil_image=initialDefaultPicture)
        return None
    return ImageInputResult(component_value=component_value)



from . import utils
from .utils import critical_section

from .template_component import template_component
from .audio_input import audio_input
from .version import VERSION as __version__
from .image_input import image_input
from .image_crop import image_crop
from .image_position_overlays import image_position_overlays
